require 'autocommit/gatherer'
require 'autocommit/git_committer'

module AutoCommit
  describe Gatherer do
    Epsilon = Gatherer::Delay * 0.2

    before :each do
      @committer = double GitCommitter
      @gatherer = Gatherer.new TmpDir, @committer
    end

    it "Triggers a delayed commit" do
      @gatherer.trigger_commit "f"
      sleep Gatherer::Delay - Epsilon
      expect(@committer).to receive :commit
      sleep 2 * Epsilon
    end

    it "Triggers twice with a large delay" do
      expect(@committer).to receive(:commit).twice
      @gatherer.trigger_commit "f"
      sleep Gatherer::Delay + Epsilon
      @gatherer.trigger_commit "g"
      sleep Gatherer::Delay + Epsilon
    end

    it "Triggers once with a small delay" do
      expect(@committer).to receive(:commit)
      @gatherer.trigger_commit "f"
      sleep 0.4 * Gatherer::Delay
      @gatherer.trigger_commit "g"
      sleep Gatherer::Delay + Epsilon
    end

    it "Triggers twice with a nice series of events" do
      expect(@committer).to receive(:commit).twice
      4.times {
        @gatherer.trigger_commit "f"
        sleep 0.4 * Gatherer::Delay
      }
      sleep 0.8 * Gatherer::Delay + Epsilon
    end

    it "Does not trigger on a Git file" do
      @gatherer.trigger_commit File.join(TmpDir, ".git/index")
      sleep Gatherer::Delay + Epsilon
    end
  end
end
